#include <stdio.h>

#include "fltkCurveEditor.h"

#include "aniUI.h"
#include "scene.h"
#include "curveDisplay.h"

#include "CurveEditor.h"
#include "Curves.h"
#include "ctrlPanel.h"

extern ctrlPanel* g_panel;

extern CurveEditor *curve_editor;
extern void CurveEditor_UpdateVariables();
extern void CurveEditor_Redraw();

extern float getCurrentTime();
extern void setCurrentTime(float);

extern CAniUI *aniUI;

extern int number_of_control_points;
extern int number_of_curve_segments;
extern float g_fps;

extern double minimum_time;
extern double time_interval;
extern double maximum_time;
	
extern double minimum_value;
extern double value_interval;
extern double maximum_value;

extern double shift_x;
extern double shift_y;

extern double new_y_minimum;
extern double new_y_maximum;

void updateCurrentlySelectedCurve(const char *name)
{
	aniUI->m_pCurveEditor->updateCurrentlySelectedCurve(name);
}

void updateMaxMinYValue(float max, float min)
{
	if (max<min) {
		float t=max;
		max=min;
		min=t;
	}

	aniUI->m_pCurveEditor->updateMaxMin(max, min);
}

void printNiceFloat(char *buf, float v, float d)
{
	int n;

	if (d>=0.1) n=1;
	else if (d>=0.01) n=2;
	else if (d>=0.001) n=3;
	else if (d>=0.0001) n=4;
	else n=5;

	switch (n) {
	case 1:
		sprintf(buf, "%.1f", v);
		break;
	case 2:
		sprintf(buf, "%.2f", v);
		break;
	case 3:
		sprintf(buf, "%.3f", v);
		break;
	case 4:
		sprintf(buf, "%.4f", v);
		break;
	case 5:
		sprintf(buf, "%.5f", v);
		break;
	}
}

// callback functions
void fltkCurveEditor::cb_max_ani_time(Fl_Widget* o, void* v)
{
	fltkCurveEditor *self=(fltkCurveEditor*)(o->user_data());
	self->setMaxTime(self->getMaxAniTimeInput());
}

void fltkCurveEditor::setMaxTime(double time)
{
	static char buf[8];
	float prevAniLength=m_fAniLength;
	curve_editor->changeAnimationLength(m_fAniLength=time);
	CurveEditor_UpdateVariables();

	if ( (m_fAniLength<m_nEnd) || (m_nEnd==prevAniLength) ){
		sprintf(buf, "%d", (int)(m_fAniLength));
		m_pEndInput->value(buf);
		m_pMaxAniTimeInput->value(buf);
		m_nEnd=m_fAniLength;
	}

	updateTimeScale();
	redraw();
	refresh();
}

void fltkCurveEditor::setEndTime(double time)
{
	static char bf[8];
	m_nEnd = time;
	sprintf(bf, "%d", (int)time);
	m_pEndInput->value(bf);
	redraw();
	refresh();
}

float fltkCurveEditor::getAnimationLength()
{
	return m_fAniLength;
}



void fltkCurveEditor::cb_scale_factor(Fl_Widget* o, void* v)
{
	fltkCurveEditor *self=(fltkCurveEditor*)(o->user_data());

	curve_editor->rescaleTime(self->getScaleFactorInput());
	self->m_fAniLength=self->m_fAniLength*self->getScaleFactorInput();
	curve_editor->changeAnimationLength(self->m_fAniLength);
	CurveEditor_UpdateVariables();

	static char buf1[16], buf2[16];

	if (self->m_fAniLength<self->m_nEnd) {
		sprintf(buf1, "%d", (int)(self->m_fAniLength));
		self->m_pEndInput->value(buf1);
		self->m_nEnd=self->m_fAniLength;
	}

	sprintf(buf2, "%.3f", self->m_fAniLength);
	self->m_pMaxAniTimeInput->value(buf2);
	self->m_nEnd=self->m_fAniLength;
	self->updateTimeScale();
	self->redraw();
	self->refresh();
}

void fltkCurveEditor::cb_max_min_value(Fl_Widget* o, void* v)
{
	fltkCurveEditor *self=(fltkCurveEditor*)(o->user_data());

	float max=self->getMaxValueInput();
	float min=self->getMinValueInput();

	updateMaxMinYValue(max, min);

	curve_editor->changeMaximumY(max);
	curve_editor->changeMinimumY(min);

	CurveEditor_UpdateVariables();
	self->updateValueScale();
	self->refresh();
	self->redraw();
}

void fltkCurveEditor::updateViewpoint()
{	
	Curve * qnx = Curves::getInstance()->getCurve(qnowx);
	qnx->forcePoint(getCurrentTime(), m_pScene->qnow.x);

	Curve * qny = Curves::getInstance()->getCurve(qnowy);
	qny->forcePoint(getCurrentTime(), m_pScene->qnow.y);

	Curve * qnz = Curves::getInstance()->getCurve(qnowz);
	qnz->forcePoint(getCurrentTime(), m_pScene->qnow.z);

	Curve * qnw = Curves::getInstance()->getCurve(qnoww);
	qnw->forcePoint(getCurrentTime(), m_pScene->qnow.w);

	Curve * ctx = Curves::getInstance()->getCurve(centerx);
	ctx->forcePoint(getCurrentTime(), m_pScene->center.x);

	Curve * cty = Curves::getInstance()->getCurve(centery);
	cty->forcePoint(getCurrentTime(), m_pScene->center.y);

	Curve * ctz = Curves::getInstance()->getCurve(centerz);
	ctz->forcePoint(getCurrentTime(), m_pScene->center.z);

	Curve * zm = Curves::getInstance()->getCurve(zoomed);
	zm->forcePoint(getCurrentTime(), m_pScene->zoom);

	Curve * vp = Curves::getInstance()->getCurve(viewpoint);
	vp->forcePoint(getCurrentTime(), 0);
	redraw();
}

void fltkCurveEditor::cb_set_viewpoint(Fl_Widget* o, void* v)
{
	fltkCurveEditor *self=(fltkCurveEditor*)(o->user_data());
	self->updateViewpoint();
}

void fltkCurveEditor::cb_wrap_curve(Fl_Widget* o, void* v)
{
	fltkCurveEditor *self=(fltkCurveEditor*)(o->user_data());
	
	curve_editor->toggleWrapControlPoints(self->m_pWrapCurveButtom->value());
	CurveEditor_UpdateVariables();

	self->refresh();
}

void fltkCurveEditor::cb_viewpoint_edit(Fl_Widget* o, void* v)
{
	fltkCurveEditor *self=(fltkCurveEditor*)(o->user_data());
	if(self->m_pViewpointEditButton->value())
	{
		self->m_pScene->viewpointEdit = true;
		self->m_pSetViewpointButton->activate();
		self->m_pScene->qdown.x = g_panel->getControlValue(qnowxid);
		self->m_pScene->qdown.y = g_panel->getControlValue(qnowyid);
		self->m_pScene->qdown.z = g_panel->getControlValue(qnowzid);
		self->m_pScene->qdown.w = g_panel->getControlValue(qnowwid);
		self->m_pScene->getView()->redraw();

	}
	else
	{
		self->m_pScene->viewpointEdit = false;
		self->m_pSetViewpointButton->deactivate();
		self->m_pScene->qdown.x = g_panel->getControlValue(qnowxid);
		self->m_pScene->qdown.y = g_panel->getControlValue(qnowyid);
		self->m_pScene->qdown.z = g_panel->getControlValue(qnowzid);
		self->m_pScene->qdown.w = g_panel->getControlValue(qnowwid);
		self->m_pScene->getView()->redraw();
	}

}


void fltkCurveEditor::cb_undo_zoom(Fl_Widget* o, void* v)
{
	fltkCurveEditor *self=(fltkCurveEditor*)(o->user_data());

	curve_editor->tryPopZoom();
	CurveEditor_UpdateVariables();

	self->updateTimeScale();
	self->updateValueScale();
	self->updateMaxMin(new_y_maximum, new_y_minimum);
	self->refresh();
	self->redraw();
}

void fltkCurveEditor::cb_set_start_end(Fl_Widget* o, void* v)
{
	fltkCurveEditor *self=(fltkCurveEditor*)(o->user_data());

	int start=atoi(self->m_pStartInput->value());
	int end=atoi(self->m_pEndInput->value());

	if (start>end) {
		int t=start;
		start=end;
		end=t;
	}

	if (start<0) start=0;
	if (end>self->m_fAniLength) 
		end=(int)(self->m_fAniLength);

	static char buf1[8], buf2[8];
	sprintf(buf1, "%d", start);
	self->m_pStartInput->value(buf1);
	sprintf(buf2, "%d", end);
	self->m_pEndInput->value(buf2);
	self->m_nStart=start;
	self->m_nEnd=end;
}

void fltkCurveEditor::setCurveButton(int val)
{
	m_pWrapCurveButtom->value(val);
}

void fltkCurveEditor::updateCurrentlySelectedCurve(const char *name) 
{
	m_pTextContainerBox->redraw();

	static char buf1[64];
	strcpy(buf1, name);
	char *ptr=strchr(buf1, '#');
	if (ptr!=NULL) *ptr=0;
	m_pSelectCurveText->label(buf1);
	m_pSelectCurveText->redraw();

	static char buf2[8];
	sprintf(buf2, "%d", number_of_control_points);
	m_pNumCtrlPointsText->label(buf2);
	m_pNumCtrlPointsText->redraw();

	static char buf3[8];
	sprintf(buf3, "%d", number_of_curve_segments);
	m_pNumCtrlSegmentsText->label(buf3);
	m_pNumCtrlSegmentsText->redraw();

	static char buf4[8];
	sprintf(buf4, "%3.2f", g_fps);
	m_pFPSText->label(buf4);
	m_pFPSText->redraw();
}

void fltkCurveEditor::updateMaxMin(float max, float min)
{
	static char max_buf[16], min_buf[16];

	sprintf(max_buf, "%.3f", max);
	sprintf(min_buf, "%.3f", min);

	m_pMaxValueInput->value(max_buf);
	m_pMinValueInput->value(min_buf);
}


float fltkCurveEditor::getStartTime()
{
	return m_nStart;
}

float fltkCurveEditor::getEndTime()
{
	return m_nEnd;
}


static char time_scale_text[32][8]={"2.0", "4.0", "6.0", "8.0", "10.0", "12.0", "14.0", "16.0", "18.0"};

void fltkCurveEditor::updateTimeScale()
{
	float stop_time_labels=maximum_time-(time_interval/2.0);
	float startx=400*shift_x/(maximum_time-minimum_time);
	int i=0;
	for (float current_time_value=minimum_time; 
				current_time_value<stop_time_labels;
				current_time_value+=time_interval, i++) {
		if (i>=32) {
			printf("Ooop!");
			break;
		}
		printNiceFloat(time_scale_text[i], current_time_value, time_interval);
		m_pTimeScale[i]->position(50+(int)(startx+400*(current_time_value-minimum_time)/(maximum_time-(minimum_time-time_interval)))-20, 405);
	}
	for (; i<32; i++) {
		m_pTimeScale[i]->position(1024, 405);
	}
}

static char value_scale_text[32][8];

void fltkCurveEditor::updateValueScale()
{
	float stop_y_labels=maximum_value-(value_interval/2.0);
	float starty=400*shift_y/(maximum_value-minimum_value);
	int i=0;
	for (float current_y_value=minimum_value; 
				current_y_value<stop_y_labels;
				current_y_value+=value_interval, i++) {
		if (i>=32) {
			printf("Ooop!");
			break;
		}
		printNiceFloat(value_scale_text[i], current_y_value, value_interval);
		m_pValueScale[i]->position(5, 400-(int)(starty+400*(current_y_value-minimum_value)/(maximum_value-(minimum_value-value_interval))));
	}
	for (; i<32; i++) {
		m_pValueScale[i]->position(5, 1024);
	}
}


void fltkCurveEditor::refresh()
{
	m_pCurveDisplay->refresh();
}

void fltkCurveEditor::redraw()
{
	m_pWindow->redraw();
}

fltkCurveEditor::fltkCurveEditor(int x, int y, int w, int h, const char *l)
{
	m_pScene = NULL;

	m_pWindow = new Fl_Window(x, y, w, h, l);

		Fl_Box *o;
		o = new Fl_Box(10, 440, 140, 20, "Currently Selected Curve:");
		o->labelsize(11);
		o->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);
		o = new Fl_Box(10, 460, 140, 20, "Number of Control Points:");
		o->labelsize(11);
		o->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);
		o = new Fl_Box(10, 480, 140, 20, "Number of Curve Segments:");
		o->labelsize(11);
		o->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);
		o = new Fl_Box(10, 500, 140, 20, "FPS:");
		o->labelsize(11);
		o->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

		m_pTextContainerBox=new textBox(145, 440, 110, 80);
		m_pSelectCurveText=new Fl_Box(145, 440, 110, 20, "");
		m_pSelectCurveText->labelsize(11);
		m_pSelectCurveText->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE|FL_ALIGN_CLIP);

		m_pNumCtrlPointsText=new Fl_Box(145, 460, 110, 20, "");
		m_pNumCtrlPointsText->labelsize(11);
		m_pNumCtrlPointsText->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE|FL_ALIGN_CLIP);

		m_pNumCtrlSegmentsText=new Fl_Box(145, 480, 110, 20, "");
		m_pNumCtrlSegmentsText->labelsize(11);
		m_pNumCtrlSegmentsText->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE|FL_ALIGN_CLIP);

		m_pFPSText=new Fl_Box(145, 500, 110, 20, "");
		m_pFPSText->labelsize(11);
		m_pFPSText->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE|FL_ALIGN_CLIP);

		m_pWrapCurveButtom=new Fl_Check_Button(20, 530, 80, 20, "Wrap Curve");
		m_pWrapCurveButtom->labelsize(12);
		m_pWrapCurveButtom->callback(cb_wrap_curve);
		m_pWrapCurveButtom->user_data((void*)this);

		m_pViewpointEditButton=new Fl_Check_Button(20, 560, 80, 20, "Edit View");
		m_pViewpointEditButton->labelsize(12);
		m_pViewpointEditButton->callback(cb_viewpoint_edit);
		m_pViewpointEditButton->user_data((void*)this);

		m_pUndoZoomButtom=new Fl_Button(120, 530, 80, 20, "Undo Zoom");
		m_pUndoZoomButtom->labelsize(12);
		m_pUndoZoomButtom->callback(cb_undo_zoom);
		m_pUndoZoomButtom->user_data((void*)this);

		m_pSetViewpointButton = new Fl_Button(120, 560, 80, 20, "Set View");
		m_pSetViewpointButton->labelsize(12);
		m_pSetViewpointButton->callback(cb_set_viewpoint);
		m_pSetViewpointButton->user_data((void*)this);


		o = new Fl_Box(260, 440, 120, 20, "Animation Length:");
		o->labelsize(11);
		o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
		o = new Fl_Box(260, 465, 120, 20, "Scaling Factor:");
		o->labelsize(11);
		o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
		o = new Fl_Box(260, 490, 120, 20, "Minimum Value:");
		o->labelsize(11);
		o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
		o = new Fl_Box(260, 515, 120, 20, "Maximum Value:");
		o->labelsize(11);
		o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);

		m_pMaxAniTimeInput = new Fl_Float_Input(350, 440, 50, 20, "");
		m_pMaxAniTimeInput->textsize(10);
		m_pMaxAniTimeInput->value("20");
		m_fAniLength=20.0;

		m_pScaleFactorInput = new Fl_Float_Input(350, 465, 50, 20, "");
		m_pScaleFactorInput->textsize(10);
		m_pScaleFactorInput->value("1.0");
		m_pMinValueInput = new Fl_Float_Input(350, 490, 50, 20, "");
		m_pMinValueInput->textsize(10);
		m_pMinValueInput->value("");
		m_pMaxValueInput = new Fl_Float_Input(350, 515, 50, 20, "");
		m_pMaxValueInput->value("");
		m_pMaxValueInput->textsize(10);


		m_pMaxAniTimeApplyButton = new Fl_Button(405, 440, 40, 20, "Apply");
		m_pMaxAniTimeApplyButton->labelsize(11);
		m_pMaxAniTimeApplyButton->callback(cb_max_ani_time);
		m_pMaxAniTimeApplyButton->user_data((void*)this);

		m_pScaleFactorApplyButton = new Fl_Button(405, 465, 40, 20, "Apply");
		m_pScaleFactorApplyButton->labelsize(11);
		m_pScaleFactorApplyButton->callback(cb_scale_factor);
		m_pScaleFactorApplyButton->user_data((void*)this);

		m_pMaxValueApplyButton = new Fl_Button(405, 490, 40, 20, "Apply");
		m_pMaxValueApplyButton->labelsize(11);
		m_pMaxValueApplyButton->callback(cb_max_min_value);
		m_pMaxValueApplyButton->user_data((void*)this);

		m_pMinValueApplyButton = new Fl_Button(405, 515, 40, 20, "Apply");
		m_pMinValueApplyButton->labelsize(11);
		m_pMinValueApplyButton->callback(cb_max_min_value);
		m_pMinValueApplyButton->user_data((void*)this);

		o = new Fl_Box(260, 540, 50, 20, "Start:");
		o->labelsize(11);
		o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);

		m_pStartInput = new Fl_Int_Input(290, 540, 40, 20, "");
		m_pStartInput->textsize(10);
		m_pStartInput->value("0");
		m_nStart=0;

		o = new Fl_Box(330, 540, 30, 20, "End:");
		o->labelsize(11);
		o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);

		m_pEndInput = new Fl_Int_Input(360, 540, 40, 20, "");
		m_pEndInput->textsize(10);
		m_pEndInput->value("20");
		m_nEnd=20;

		m_pSetStartEndButton = new Fl_Button(405, 540, 40, 20, "Set");
		m_pSetStartEndButton->labelsize(11);
		m_pSetStartEndButton->callback(cb_set_start_end);
		m_pSetStartEndButton->user_data((void*)this);

		// prepare time scale texts
		int i;
		for (i=0; i<32; i++) {
			if (i<9)
				m_pTimeScale[i]=new Fl_Box(70+40*i, 405, 40, 20, time_scale_text[i]);
			else 
				m_pTimeScale[i]=new Fl_Box(1024, 405, 40, 20, time_scale_text[i]);
			m_pTimeScale[i]->labelsize(11);
			m_pTimeScale[i]->align(FL_ALIGN_INSIDE);
		}

		// prepare value scale texts
		for (i=0; i<32; i++) {
				m_pValueScale[i]=new Fl_Box(5, 1024, 40, 20, value_scale_text[i]);
			m_pValueScale[i]->labelsize(11);
			m_pValueScale[i]->align(FL_ALIGN_INSIDE|FL_ALIGN_RIGHT);
		}

		// Curve display
		m_pCurveDisplay = new curveDisplay(45, 5, 400, 400, "", this);
		m_pCurveDisplay->box(FL_DOWN_FRAME);
		curve_editor->specifyWindowSize(400, 400);
	m_pViewpointEditButton->value(0);
	m_pSetViewpointButton->deactivate();
	m_pWindow->end();
}

void fltkCurveEditor::resetCurveMenu(Object *obj) 
{ 
	m_pCurveDisplay->resetCurveMenu(obj);
}

void fltkCurveEditor::setScene(Scene *scn)
{
	m_pScene = scn;
}

void fltkCurveEditor::show()
{
	m_pWindow->show();
	m_pCurveDisplay->show();
}

void fltkCurveEditor::hide()
{
	m_pCurveDisplay->hide();
	m_pWindow->hide();
}