//
// ctrlPanel.h
//
// header file for control panel class
//
//

#ifndef __CTRLPANEL_H__
#define __CTRLPANEL_H__

// if you want to see 26 ugly warning message, just comment this line. :>
#pragma warning(disable:4786)

#include <FL/Fl.H>
#include <FL/Fl_Widget.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Tabs.H>

#include <FL/Fl_Value_Slider.H>
#include <FL/Fl_Light_Button.H>
#include <FL/Fl_Choice.H>
#include <FL/Fl_Menu_Item.H>

#include <list>
#include <vector>
#include <map>

#include <string.h>

using namespace std;

#define INIT_SPACING		5
#define WIDGET_HEIGHT		18
#define YPIXELS_PER_WIDGET	22
#define XPIXELS_PER_WIDGET	400

#define SLIDER_WIDTH		250
#define CHECKBUTTON_WIDTH	16
#define MENU_WIDTH	160

//#define WINDOW_HEIGHT		800			// use Fl::h() instead (DWJ 4/00)

#define FONT_SIZE			12

#define CURVE_MODE			1

extern int g_mode;
extern bool g_viewCurve;
extern bool g_viewBezier;

class CAniUI;
class Object;

class params {
public:
	params(void *ptr, int n) { p=ptr; id=n; };

	void *p;
	int id;
};

class value_pair {
public:
	value_pair(float v, float cv, void *p, char *s, bool b, int g=0, bool t=false) 
			{ value=v; c_value=cv; ptr=p; strcpy(name, s); curve=b; toggle=t; group_id=g; };

	float value;		// value is for control
	float c_value;		// c_value is for curve
	int group_id;		// index of the tab it should be grouped under
	void  *ptr;			// ptr is used to store user data
	char name[64];
	bool curve;
	bool toggle;
};

typedef pair<const char *,Fl_Group *> group_name_pair;
typedef vector<group_name_pair> grouplist;

class ctrlPanel : public Fl_Window {
public:
	ctrlPanel(const char* title=0);
	ctrlPanel(int h, const char* title=0);
	ctrlPanel(int x, int y, const char* title=0);
	ctrlPanel(int x, int y, int h, const char* title=0);

	~ctrlPanel();

	void makeCurrent();

	void setUI(CAniUI* ui) { m_pUI=ui; };
	CAniUI* getUI() { return m_pUI; };

	typedef map<int, value_pair *, less<int> > valueMap;
	typedef valueMap::const_iterator map_iterator;

	ctrlPanel::map_iterator begin(void) const;
	ctrlPanel::map_iterator end(void) const;

	float getControlValue(int id) { 
		float t;
		map_iterator j=values.find(id);
		if (j!=values.end()) {
			if(g_mode==CURVE_MODE)
				return ((*j).second)->c_value; 
			else
				return t=((*j).second)->value; 
		}
		else return 0;
	};
	void setControlValue(int id, float v) { 
		values[id]->value=v; 
	};
	void setCurveValue(int id, float v) { 
		values[id]->c_value=v; 
	};
	char *getCurveName(int id) { 
		map_iterator j=values.find(id);
		if (j!=values.end()) return ((*j).second)->name; 
		else return NULL;
	};
	void toggleCurve(int id) {
		map_iterator j=values.find(id);
		if (j!=values.end()) {
			((*j).second)->toggle=!((*j).second)->toggle; 
		}
	}
	void *getPtr(int id) {
		map_iterator j=values.find(id);
		if (j!=values.end()) {
			return ((*j).second)->ptr;
		}
		else return NULL;
	}

	void completeInitialization();

	void group(char *);
	void curve(int id, char *name, float min, float max, float start, int groupID);
	void scale(int id, char *name, float min, float max, float res, float start, int groupID);
	void checkbox(int id, char *name, int state, int groupID);
	void menu(int id, char *name, int start, Fl_Menu_Item* menu, int groupID);
	void ptr(int id, void *ptr);

	void setObject(Object *obj) { m_pObject=obj; };
	Object *getObject() { return m_pObject; };

	grouplist::iterator getGroupBegin() { return m_vGroups.begin();};
	grouplist::iterator getGroupEnd(){ return m_vGroups.end();};
	int	getGroupCount() {return m_vGroups.size();};
private:
	int m_nTargetHeight;
	int m_nWidgetPerColumn;

	CAniUI *m_pUI;
	Object *m_pObject;

	Fl_Tabs * m_Tabs;
	grouplist m_vGroups;
	
	valueMap values;

	// callback functions
	static void cb_exit(Fl_Widget* o, void* v);
	static void cb_slides(Fl_Widget *o, void *v);
	static void cb_check(Fl_Widget *o, void *v);
	static void cb_menu(Fl_Widget *o, void *v);
};

#endif