#include "modeler.h"

#include "modelerdll.h"

int g_quality = 50;
int g_drawmode = 1;

FILE* g_rayfile = NULL;

GLfloat mat_ambient[4];
GLfloat mat_diffuse[4];
GLfloat mat_specular[4];
GLfloat mat_shininess;

/* DllMain
 * 
 * every DLL must provide DllMain.  ours just stores the instance handle,
 * so we can use it to create a window later.
 */

#ifdef WIN32
BOOL WINAPI DllMain( HINSTANCE hInst, DWORD dwReason, LPVOID reserved )
{
//    g_hInstance = hInst;
    return 1;
}
#endif

void setdrawmode( int drawmode, char *rayfilename ) {

g_drawmode = drawmode;

if ( drawmode & MODE_HIGH_QUALITY )	g_quality = 50;
if ( drawmode & MODE_MED_QUALITY )	g_quality = 20;
if ( drawmode & MODE_LOW_QUALITY )	g_quality = 10;
if ( drawmode & MODE_POOR_QUALITY )	g_quality = 4;

// g_rayfile is closed if it is open, no matter what drawing mode

if (g_rayfile != NULL) {
	fclose( g_rayfile );
	g_rayfile = NULL;
	}

// if a ray file name is provided, the file is opened, no matter what drawing mode

if ( rayfilename != NULL ) {
	g_rayfile = fopen( rayfilename, "w" );
	if ( g_rayfile != NULL ) {
		fprintf( g_rayfile, "SBT-raytracer 1.0\n\n" );
		fprintf( g_rayfile, "camera { fov=30; }\n\n" );
		fprintf( g_rayfile, "directional_light { direction=(-1,-1,-1); color=(0.7,0.7,0.7); }\n\n" );
		}
	}
}


/* ambient_color
 *
 * models can call this function to set the ambient color for drawing
 * primitives.  this only has an effect if lighting is being used
 * ("normal" drawing mode).
 */

void ambient_color( double r, double g, double b )
{
    mat_ambient[0] = (GLfloat)r;
    mat_ambient[1] = (GLfloat)g;
    mat_ambient[2] = (GLfloat)b;
    mat_ambient[3] = 1.0f;
    
    if ( g_drawmode & MODE_NORMAL )
		glMaterialfv( GL_FRONT_AND_BACK, GL_AMBIENT, mat_ambient );
}

/* diffuse_color
 *
 * models can call this function to set the diffuse color for drawing
 * primitives.  this is also the color used for drawing in wireframe
 * and flat-shaded modes.
 */

void diffuse_color( double r, double g, double b )
{
    mat_diffuse[0] = (GLfloat)r;
    mat_diffuse[1] = (GLfloat)g;
    mat_diffuse[2] = (GLfloat)b;
    mat_diffuse[3] = 1.0f;
	
    if ( g_drawmode & MODE_NORMAL )
		glMaterialfv( GL_FRONT_AND_BACK, GL_DIFFUSE, mat_diffuse );
    else
		glColor3d( r, g, b );
}

/* specular_color
 *
 * models can call this function to set the specular color for drawing
 * primitives.  this only has an effect if lighting is being used
 * ("normal" drawing mode).
 */

void specular_color( double r, double g, double b )
{
    mat_specular[0] = (GLfloat)r;
    mat_specular[1] = (GLfloat)g;
    mat_specular[2] = (GLfloat)b;
    mat_specular[3] = 1.0f;

    if ( g_drawmode & MODE_NORMAL )
		glMaterialfv( GL_FRONT_AND_BACK, GL_SPECULAR, mat_specular );
}

/* shininess
 *
 * models can call this function to set the specular shininess for
 * drawing primitives.  this only has an effect if lighting is being
 * used ("normal" drawing mode).
 */

void shininess( double s )
{
    mat_shininess = (GLfloat)s;
    
    if ( g_drawmode & MODE_NORMAL )
    {
		glMaterialf( GL_FRONT, GL_SHININESS, mat_shininess );
    }
}